.jp-VarInspector {
    flex-direction: column;
    overflow: auto;
    font-size: var(--jp-ui-font-size1);
  }
  
  .jp-VarInspector-table {
    border-collapse: collapse;
    margin: auto;
    width: 100%;
    color: var(--jp-content-font-color1);
  }
  
  .jp-VarInspector-table td,
  .jp-VarInspector-table thead {
    border: 1px solid;
    border-color: var(--jp-layout-color2);
    padding: 8px;
  }
  
  .jp-VarInspector-content tr:hover {
    background-color: var(--jp-layout-color2);
    cursor: default;
  }
  
  .jp-VarInspector-table thead {
    font-size: var(--jp-ui-font-size0);
    text-align: center;
    background-color: var(--jp-layout-color2);
    color: var(--jp-ui-font-color1);
    font-weight: 600;
    letter-spacing: 1px;
    text-transform: none;
  }
  
  .jp-VarInspector-title {
    font-size: var(--jp-ui-font-size1);
    color: var(--jp-content-font-color1);
    text-align: left;
    padding-left: 10px;
  }
  
  .jp-VarInspector-deleteButton {
    text-align: center;
    width: 1em;
  }
  
  .jp-VarInspector-deleteButton:hover {
    text-shadow: 0 0 0 red;
    background-color: var(--jp-layout-color3);
  }
  
  .jp-VarInspector-varName {
    font-weight: 600;
  }
  
  .jp-VarInspector-varName:hover {
    background-color: var(--jp-layout-color3);
  }
  
  /* Style buttons */
  .btn {
    color: black; /* White text */
    font-family: verdana;
    padding: 8px 12px; /* Some padding */
    font-size: 14px; /* Set a font size */
    cursor: pointer; /* Mouse pointer on hover */
  }
  
  /* Darker background on mouse-over */
  .btn:hover {
    background-color: coral;
  }
  
  .small-btn {
    color: black; /* White text */
    font-size: 12px; /* Set a font size */
    cursor: pointer; /* Mouse pointer on hover */
  }

    /* Darker background on mouse-over */
  .small-btn :hover {
    background-color: coral;
  }
  
  .tomato-text {
    color: tomato;
    cursor: pointer;
    font-size: 16px;
  }

  .plain-text {
    font-family:'verdana';
    font-size: 16px;
  }

  .padded-text {
    font-family:'verdana';
    padding-left:5em;
    box-decoration-break: clone;
    font-size: 16px;
  }

  .box {
    margin-right: auto;
    margin-top: 20px;
    box-shadow: 1px 0px;
    border: 5px double; 
    padding: 10px 10px 10px 10px;
    overflow-x: scroll;
  }

  .padded-div {
    padding-left: 5em;
    text-indent: -5em;
  }

.my-apodWidget {
    display: flex;
    flex-direction: column;
    align-items: center;
    overflow-x: auto;
  }
  
  .myButton {
    background: black;
  }
  
  .jp-Button.minimal {
    color: black;
  }
  
  .jp-ReactWidget {
    color: var(--jp-ui-font-color1);
    background: var(--jp-layout-color1);
    padding: 20px;
    overflow: scroll;
  }
  
