"use strict";(self.webpackChunkjlmc=self.webpackChunkjlmc||[]).push([[57,479],{90057:(e,u,n)=>{n.r(u),n.d(u,{useImmer:()=>s,useImmerReducer:()=>a});var r=n(29573),c=n.n(r),t=n(56271);function s(e){var u=(0,t.useState)(e),n=u[1];return[u[0],(0,t.useCallback)((function(e){n("function"==typeof e?c()(e):e)}),[])]}function a(e,u,n){var r=(0,t.useMemo)((function(){return c()(e)}),[e]);return(0,t.useReducer)(r,u,n)}}}]);